@@ -1,10 +1,16 @@
-if __name__ == "__main__":
-    import sys
-
-    sys.path.insert(0, ".")
-    from demo.demo_00 import demo
-
-    demo()
+#    Copyright © 2021 Andrei Puchko
+#
+#    Licensed under the Apache License, Version 2.0 (the "License");
+#    you may not use this file except in compliance with the License.
+#    You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS,
+#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#    See the License for the specific language governing permissions and
+#    limitations under the License.
 
 
 from q2report.q2printer.q2printer import Q2Printer
@@ -12,6 +18,8 @@ from q2report.q2printer.docx_parts import docx_parts
 from q2report.q2utils import num, int_, reMultiSpaceDelete
 import zipfile
 import base64
+from .rich_text_parser import RichTextParser, css_color_to_rgb
+import re
 
 points_in_mm = 2.834645669
 points_in_cm = num(points_in_mm) * num(10)
@@ -99,10 +107,9 @@ class Q2PrinterDocx(Q2Printer):
         page_margin_top=1,
         page_margin_right=1,
         page_margin_bottom=1,
-        last_page=False,
     ):
 
-        self.close_docx_page(last_page)
+        self.close_docx_page()
 
         super().reset_page(
             page_width,
@@ -115,7 +122,7 @@ class Q2PrinterDocx(Q2Printer):
 
         self.page_params = True
 
-    def close_docx_page(self, last_page):
+    def close_docx_page(self, last_page=False):
         self.close_docx_table()
         if self.page_params:
             header_ref_xml = ""
@@ -167,8 +174,8 @@ class Q2PrinterDocx(Q2Printer):
                             {header_ref_xml}
                             {footer_ref_xml}
                             <w:type w:val="nextPage"/>
-                            <w:pgSz 
-                                w:w="{self.page_width * twip_in_cm}" 
+                            <w:pgSz
+                                w:w="{self.page_width * twip_in_cm}"
                                 w:h="{self.page_height * twip_in_cm}"/>
                             <w:pgMar w:gutter="0" w:header="708" w:footer="708"
                                     w:top="{self.page_margin_top * twip_in_cm}"
@@ -210,7 +217,7 @@ class Q2PrinterDocx(Q2Printer):
                         <w:tblLayout w:type="fixed"/>
                         <w:tblInd w:w="28" w:type="dxa"/>
                         <w:tblW w:w="{round(sum(int_(x * twip_in_cm) for x in self._cm_columns_widths))}"
-                             w:type="dxa"/>
+                            w:type="dxa"/>
                         <w:tblCellMar>
                             <w:top w:w="28" w:type="dxa"/>
                             <w:left w:w="28" w:type="dxa"/>
@@ -229,8 +236,8 @@ class Q2PrinterDocx(Q2Printer):
         super().render_rows_section(rows_section, style, outline_level)
         spanned_cells_first_column_cell = {}
         spanned_cells_empty_column_cell = {}
-        if rows_section["role"] == "table_header":
-            self.reset_columns()
+        # if rows_section["role"] == "table_header":
+        #     self.reset_columns()
 
         row_section_xml = []
 
@@ -272,9 +279,9 @@ class Q2PrinterDocx(Q2Printer):
                         for tmp_span_col in range(int_(col_span)):
                             span_key = f"{tmp_span_row+row},{tmp_span_col+col}"
                             if tmp_span_row + row != row and tmp_span_col + col == col:
-                                spanned_cells_first_column_cell[
-                                    span_key
-                                ] = f'{merge_str} <w:vMerge w:val="continue"/>'
+                                spanned_cells_first_column_cell[span_key] = (
+                                    f'{merge_str} <w:vMerge w:val="continue"/>'
+                                )
                             if tmp_span_col + col > col:
                                 spanned_cells_empty_column_cell[span_key] = ""
 
@@ -288,6 +295,7 @@ class Q2PrinterDocx(Q2Printer):
                         cell_width,
                         merge_str,
                         self.get_cell_images(cell_data),
+                        rows_section["row_height"][row],
                     )
                 )
 
@@ -317,28 +325,37 @@ class Q2PrinterDocx(Q2Printer):
             cell_images_list.append(docx_parts["image"] % locals())
         return "\n".join(cell_images_list)
 
-    def open_table_row(self, row, rows):
+    def open_table_row(self, row, rows_section):
         row_xml = ""
         row_xml += "\n\t<w:tr>"
         row_xml += "\n\t\t<w:trPr>"
-        if rows["role"] == "table_header":
+        if rows_section["role"] == "table_header":
             row_xml += "<w:tblHeader/>"
-        if rows["docx_height"][row]:
-            row_xml += f'\n\t\t\t<w:trHeight w:val="{rows["docx_height"][row]*twip_in_cm}" w:hRule="exact"/>'
+        ##################################################
+        height = rows_section["row_height"][row]
+        if row in rows_section["auto_height_rows"]:
+            height = 0
+        if rows_section["min_row_height"][row] != 0 and rows_section["max_row_height"][row] == 0:
+            row_xml += f'\n\t\t\t<w:trHeight  w:val="{int(rows_section["min_row_height"][row]*twip_in_cm)}" w:hRule="atLeast"/>'
+        elif rows_section["min_row_height"][row] == 0 and rows_section["max_row_height"][row] != 0:
+            row_xml += f'\n\t\t\t<w:trHeight w:val="{int(rows_section["max_row_height"][row]*twip_in_cm)}" w:hRule="exact"/>'
+        elif rows_section["row_height"][row] == 0 and row in rows_section["hidden_rows"]:
+            row_xml += '\n\t\t\t<w:trHeight w:val="0" w:hRule="exact"/>'
+        ##################################################
         row_xml += "\n\t\t</w:trPr>"
         return row_xml
 
     def close_table_row(self):
         return "\n\t</w:tr>"
 
-    def add_table_cell(self, cell_style, cell_text, cell_width, merge_str, images=[]):
+    def add_table_cell(self, cell_style, cell_text, cell_width, merge_str, images=[], row_height=0):
         borders = self.get_cell_borders(cell_style)
         margins = self.get_cell_paddings(cell_style)
-        para_params = self.get_paragraph_params(cell_style)
+        para_params = self.get_paragraph_params(cell_style, row_height)
         para_text = self.get_paragraph_text(cell_style, cell_text, para_params)
         valign = self.get_vertical_align(cell_style)
 
-        # self.document.append(
+        shd = self.get_cell_background(cell_style)
         return f"""
                 <w:tc>
                     <w:tcPr>
@@ -347,6 +364,7 @@ class Q2PrinterDocx(Q2Printer):
                         {merge_str}
                         {borders}
                         {margins}
+                        {shd}
                     </w:tcPr>
                     <w:p>
                         {para_params}
@@ -356,66 +374,88 @@ class Q2PrinterDocx(Q2Printer):
                 </w:tc>
         """
 
+    def get_cell_background(self, cell_style):
+        if style := cell_style.get("background"):
+            return f'<w:shd w:fill="{css_color_to_rgb(style)[2:]}" />'
+        else:
+            return ""
+
     def get_paragraph_text(self, cell_style, cell_text, para_params):
-        cell_text = cell_text.replace("\n", "")
-        cell_text = cell_text.replace("\r", "")
+        # Normalize whitespace
+        cell_text = cell_text.replace("\r\n", "\n").replace("\r", "\n")
         cell_text = reMultiSpaceDelete.sub(" ", cell_text)
+        cell_text = re.sub(r"(?i)<br\s*/?>\s*", "<br/>", cell_text.strip())
+
+        # Get base font size from cell style and convert to twips (×2)
+        base_fontsize = float(str(cell_style.get("font-size", "10pt")).replace("pt", ""))
+        base_fontsize_twips = int(base_fontsize * 2)
+
+        fontfamily = cell_style.get("font-family", "Calibri")
+
+        # Use RichTextParser
+        parser = RichTextParser(fontfamily, base_fontsize)
+        parser.feed(cell_text, cell_style)
+        runs = parser.get_runs()
+
         para_text = []
-        if "font-weight" in cell_style and cell_style["font-weight"] == "bold":
-            cell_text = f"<b>{cell_text}</b>"
-        bold = ""
-        ital = ""
-        undl = ""
-        fontsizemod = fontsize = num(cell_style["font-size"].replace("pt", "")) * 2
-        fontfamily = cell_style["font-family"]
-        for x in cell_text.split("<"):
-            if ">" in x:
-                stl = x.split(">")[0].upper().strip().replace(" ", "")
-                if "B" == stl:
-                    bold = "<w:b/>"
-                elif "/B" == stl:
-                    bold = ""
-                elif "I" == stl:
-                    ital = "<w:i/>"
-                elif "/I" == stl:
-                    ital = ""
-                elif "U" == stl:
-                    undl = """<w:u w:val="single"/>"""
-                elif "/U" == stl:
-                    undl = ""
-                elif "/FONT" in stl:
-                    fontsizemod = fontsize
-                # elif "FONTSIZE=" in stl:
-                #     fontsizemod = grid.getFontSizeMod(fontsize / 2, stl.split("=")[1]) * 2
-                elif "BR/" == stl or "BR" == stl:
-                    para_text.append(f"""</w:p><w:p>{para_params}""")
-                    # para_text.append(self.get_paragraph_params(cell_style))
-                x = x.split(">")[1]
-            if x:
+        for run in runs:
+            if run == "<br/>":
+                para_text.append(f"</w:p><w:p>{para_params}")
+                continue
+
+            # Extract run content and style
+            text_match = re.search(r"<t.*?>(.*?)</t>", run, re.DOTALL)
+            style_match = re.search(r"<rPr>(.*?)</rPr>", run, re.DOTALL)
+            if text_match:
+                text_content = text_match.group(1)
+                style_content = style_match.group(1) if style_match else ""
+
+                # Build DOCX run properties
+                docx_rPr = [f'<w:rFonts w:ascii="{fontfamily}" w:hAnsi="{fontfamily}" w:cs="{fontfamily}"/>']
+
+                # Font size: use override from style, fallback to base
+                sz_match = re.search(r'<sz val="([\d\.]+)"', style_content)
+                if sz_match:
+                    sz_twips = int(float(sz_match.group(1)) * 2)
+                else:
+                    sz_twips = base_fontsize_twips
+                docx_rPr.append(f'<w:sz w:val="{sz_twips}"/>')
+
+                # Styles
+                if "<b/>" in style_content:
+                    docx_rPr.append("<w:b/>")
+                if "<i/>" in style_content:
+                    docx_rPr.append("<w:i/>")
+                if "<u/>" in style_content:
+                    docx_rPr.append('<w:u w:val="single"/>')
+
+                color_match = re.search(r'<color rgb="([A-F0-9]+)"/>', style_content)
+                if color_match:
+                    docx_rPr.append(f'<w:color w:val="{color_match.group(1)[2:]}" />')  # strip "FF" prefix
+
                 para_text.append(
-                    f"""
-                    <w:r>
-                        <w:rPr>
-                            <w:rFonts w:ascii="{fontfamily}" w:hAnsi="{fontfamily}" w:cs="{fontfamily}"/>
-                            <w:sz w:val="{fontsizemod}"/>
-                            {bold}{ital}{undl}
-                        </w:rPr>
-                        <w:t xml:space="preserve">{x}</w:t>
-                    </w:r>
-                    """
+                    f"""<w:r>
+                            <w:rPr>{''.join(docx_rPr)}</w:rPr>
+                            <w:t xml:space="preserve">{text_content}</w:t>
+                        </w:r>"""
                 )
+
         return "".join(para_text)
 
-    def get_paragraph_params(self, cell_style):
+    def get_paragraph_params(self, cell_style, row_height):
+        font_size = float(str(cell_style.get("font-size", "10pt")).replace("pt", ""))
+        sz_twips = min(int(float(row_height) * 28.346456692913385), font_size) * 2
         paragraph = f"""
-<w:pPr>
-\t{self.get_horizontal_align(cell_style)}
-\t<w:widowControl w:val="0"/>
-\t<w:adjustRightInd w:val="0"/>
-\t<w:autoSpaceDE w:val="0"/>
-\t<w:autoSpaceDN w:val="0"/>
-\t<w:spacing w:before="0" w:after="0" w:lineRule="atLeast" w:line="0"/>
-</w:pPr>"""
+                    <w:pPr>
+                    \t{self.get_horizontal_align(cell_style)}
+                    \t<w:widowControl w:val="0"/>
+                    \t<w:adjustRightInd w:val="0"/>
+                    \t<w:autoSpaceDE w:val="0"/>
+                    \t<w:autoSpaceDN w:val="0"/>
+                    \t<w:spacing w:before="0" w:after="0" w:lineRule="atLeast" w:line="0"/>
+                    <w:rPr><w:sz w:val="{sz_twips}"/><w:szCs w:val="{sz_twips}"/></w:rPr>
+                    </w:pPr>
+                    """
         return paragraph
 
     def get_vertical_align(self, cell_style):
@@ -442,11 +482,16 @@ class Q2PrinterDocx(Q2Printer):
         border_width = cell_style["border-width"].split(" ")
         while len(border_width) < 4:
             border_width += border_width
+
+        border_color = "auto"
+        if color := cell_style.get("border-color"):
+            border_color = css_color_to_rgb(color)[2:]
+
         borders = []
         borders.append("<w:tcBorders>\n")
         for index, side in enumerate(("top", "right", "bottom", "left")):
             if int_(border_width[index]):
-                borders.append(f'\t\t\t<w:{side} w:val="single" w:color="auto" w:space="0"')
+                borders.append(f'\t\t\t<w:{side} w:val="single" w:color="{border_color}" w:space="0"')
                 borders.append(f'\t\t\t\tw:sz="{int_(border_width[index])*10}"/>')
         borders.append("</w:tcBorders>\n")
         return "\n".join(borders)
