@@ -1,10 +1,16 @@
-if __name__ == "__main__":
-    import sys
-
-    sys.path.insert(0, ".")
-    from demo.demo_00 import demo
-
-    demo()
+#    Copyright © 2021 Andrei Puchko
+#
+#    Licensed under the Apache License, Version 2.0 (the "License");
+#    you may not use this file except in compliance with the License.
+#    You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS,
+#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#    See the License for the specific language governing permissions and
+#    limitations under the License.
 
 
 from q2report.q2printer.q2printer import Q2Printer
@@ -35,7 +41,11 @@ class Q2PrinterHtml(Q2Printer):
 
         html += "\n".join(self.html)
         html += "\n\n\t</body>" "\n</html>"
-        self._OF.write(html)
+        if isinstance(self.output_file, str):
+            with open(self.output_file, "w", encoding="utf8") as f:
+                f.write(html)
+        else:
+            self.output_file.write(html.encode())
         return super().save()
 
     def reset_page(self, **args):
@@ -74,21 +84,30 @@ class Q2PrinterHtml(Q2Printer):
             self.style[style_text] = f"css{len(self.style)}"
         return self.style[style_text]
 
-    def render_rows_section(self, rows, style, outline_level):
-        super().render_rows_section(rows, style, outline_level)
-        row_count = len(rows["heights"])
+    def render_rows_section(self, rows_section, style, outline_level):
+        super().render_rows_section(rows_section, style, outline_level)
+        row_count = len(rows_section["heights"])
         spanned_cells = []
-        if rows["role"] == "table_header":
+        if rows_section["role"] == "table_header":
             self.reset_columns()
             self.html.append("\t<thead>")
         for row in range(row_count):
-            self.html.append("\t<tr>")
+            height = rows_section["row_height"][row]
+            if row in rows_section["auto_height_rows"]:
+                height = 0
+
+            if height != 0:
+                self.html.append(f'\t<tr style="height: {height}cm;">')
+            elif rows_section["row_height"][row] == 0 and row in rows_section["hidden_rows"]:
+                self.html.append('\t<tr  style="visibility:collapse">')
+            else:
+                self.html.append("\t<tr>")
+
             for col in range(self._columns_count):
                 key = f"{row},{col}"
                 if key in spanned_cells:
                     continue
-                cell_data = rows.get("cells", {}).get(key, {})
-                # cell_text = cell_data.get("data", "&nbsp;")
+                cell_data = rows_section.get("cells", {}).get(key, {})
                 row_span = cell_data.get("rowspan", 1)
                 col_span = cell_data.get("colspan", 1)
                 cell_style = cell_data.get("style", {})
@@ -106,13 +125,14 @@ class Q2PrinterHtml(Q2Printer):
                     span_text = " "
                 self.html.append(f'\t\t<td class="{style_index}" {span_text}>{cell_text}</td>')
             self.html.append("\t</tr>")
-        if rows["role"] == "table_header":
+        if rows_section["role"] == "table_header":
             self.html.append("\t</thead>")
         # if rows["role"] == "table_footer":
         #     self.html.append("<thead></thead>")
 
     def render_cell_images(self, cell_data):
-        cell_text = cell_data.get("data", "&nbsp;")
+        # cell_text = cell_data.get("data", "&nbsp;")
+        cell_text = cell_data.get("data", "")
         if cell_data.get("images"):
             for x in cell_data.get("images"):
                 image = x["image"]
@@ -129,5 +149,5 @@ class Q2PrinterHtml(Q2Printer):
         return cell_text
 
     def show(self):
-        # print(f"file://{os.path.abspath(self.output_file)}")
-        webbrowser.open_new_tab(f"file://{os.path.abspath(self.output_file)}")
+        if isinstance(self.output_file, (str, bytes, os.PathLike, int)):
+            webbrowser.open_new_tab(f"file://{os.path.abspath(self.output_file)}")
