@@ -1,45 +1,28 @@
-if __name__ == "__main__":
-    import sys
-
-    sys.path.insert(0, ".")
-    from demo.demo_00 import demo
-
-    demo()
+#    Copyright © 2021 Andrei Puchko
+#
+#    Licensed under the Apache License, Version 2.0 (the "License");
+#    you may not use this file except in compliance with the License.
+#    You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS,
+#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#    See the License for the specific language governing permissions and
+#    limitations under the License.
 
 
 from q2report.q2printer.q2printer import Q2Printer
 from q2report.q2printer.xlsx_parts import xlsx_parts
-from q2report.q2utils import num, int_, reMultiSpaceDelete
+from q2report.q2utils import num, int_, reMultiSpaceDelete, reDecimal, reNumber
+from .rich_text_parser import RichTextParser, css_color_to_rgb
 
 import zipfile
 import re
 import base64
 
-reSpaces = re.compile(r"\s*", re.IGNORECASE)
-reFormula = re.compile(r"\[[^]^[]+\]")
-reFloat = re.compile(r"[-+]?\d*\.\d+|\d+")
-reHtmlTagBr = re.compile(r"<\s*/*BR\s*/*\s*>", re.IGNORECASE)
-reHtmlTagFontSize = re.compile(r"\s*font\s*size\s*\=", re.IGNORECASE)
-reHtmlTagFontEnd = re.compile(r"\<\s*/\s*font\s*>", re.IGNORECASE)
-reFontSize = re.compile(r"\<\s*font\s+size\\s*=\s*[+-]*\s*[0-9]*\s*\>", re.IGNORECASE)
-reSignAndSize = re.compile(r"[+-]*\s*[0-9]+")
-rePixMap = re.compile(r"zzPixmap\s*\(\s*.*\s*\)")
-reIsoDate = re.compile(r"^(?P<date>(\d\d\d\d\-((0[1-9])|(1(0|1|2))))\-((0[1-9])|(1\d)|(2\d)|(3(0|1))))$")
-reFontModifiers = re.compile(r"\<(\/*b)|(br/)|(font)|(\/*u)|(\/*i)\>", re.IGNORECASE)
-
-
-reSubFontSize = re.compile(r"(<(\s*font\s+size\s*=\s*[+-]*\s*\d+\s*)>)", re.IGNORECASE)
-reSubFontEnd = re.compile(r"(<(\s*\/\s*font\s*)>)", re.IGNORECASE)
-reSubBold = re.compile(r"(<(\s*b\s*)>)", re.IGNORECASE)
-reSubBoldEnd = re.compile(r"(<(\s*\/\s*b\s*)>)", re.IGNORECASE)
-reSubItalic = re.compile(r"(<(\s*i\s*)>)", re.IGNORECASE)
-reSubItalicEnd = re.compile(r"(<(\s*\/\s*i\s*)>)", re.IGNORECASE)
-reSubUnderline = re.compile(r"(<(\s*u\s*)>)", re.IGNORECASE)
-reSubUnderlineEnd = re.compile(r"(<(\s*\/\s*u\s*)>)", re.IGNORECASE)
-
-
-reUnderLineBegin = re.compile(r"<\s*u\s*>", re.IGNORECASE)
-reUnderLineEnd = re.compile(r"<\s*/\s*U\s*>", re.IGNORECASE)
+reHtmlTagBr = re.compile(r"<\s*/*BR\s*/*\s*>\s*", re.IGNORECASE)
 
 cm_2_inch = num(2.5396)
 points_in_mm = num(2.834645669)
@@ -54,10 +37,14 @@ class Q2PrinterXlsx(Q2Printer):
         self.current_sheet = {}
         self.sheet_current_row = 1
 
-        self.fonts = ["""<sz val="11"/><name val="Calibri"/>"""]
-        self.fills = ["""<patternFill patternType="none"/>"""]
+        self.fonts = ["""<sz val="10"/><name val="Calibri"/>"""]
+        self.fills = [
+            """<fill><patternFill patternType="none"/></fill>""",
+            """<fill><patternFill patternType="none"/></fill>""",
+        ]
         self.borders = ["""<left/><right/><top/><bottom/><diagonal/>"""]
         self.cell_xfs = ["""<xf borderId="0" fillId="0" fontId="0" numFmtId="0" xfId="0"/>"""]
+        self.num_fmts = []
         self.xmlImageList = []
         self.images_size_list = []
 
@@ -94,7 +81,7 @@ class Q2PrinterXlsx(Q2Printer):
         wb_content_types = []
 
         for img in range(0, len(self.xmlImageList)):
-            zipf.writestr(r"xl\media\image%s.png" % (img + 1), base64.b64decode(self.xmlImageList[img]))
+            zipf.writestr("xl/media/image%s.png" % (img + 1), base64.b64decode(self.xmlImageList[img]))
             wb_images.append(xlsx_parts["images"] % ((img + 1), (img + 1)))
 
         for x in range(0, len(self.xlsx_sheets)):
@@ -132,31 +119,15 @@ class Q2PrinterXlsx(Q2Printer):
             )
             wb_workbook_rels.append(xlsx_parts["xl/_rels/workbook.xml.rels-line"] % (x + 9, x + 1))
 
-            #             sheet_data = "".join(
-            #                 """
-            # <row r="%s" customHeight="1" ht="%s" outlineLevel="%s" collapsed="false"> %s \n
-            # </row>"""
-            #                 % (
-            #                     z + 1,
-            #                     self.xlsx_sheets[x]["sheetData"][z]["height"],
-            #                     self.xlsx_sheets[x]["sheetData"][z]["outline_level"],
-            #                     "".join(self.xlsx_sheets[x]["sheetData"][z]["cells"]),
-            #                 )
-            #                 for z in range(len(self.xlsx_sheets[x]["sheetData"]))
-            #             )
-
-            sheet_data = "".join(
-                f"""
-                    <row
-                    \tr="{z+1}"
-                    \tcustomHeight="1" ht="{self.xlsx_sheets[x]["sheetData"][z]["height"]}"
-                    \toutlineLevel="{self.xlsx_sheets[x]["sheetData"][z]["outline_level"]}"
-                    collapsed="false"
-                    >
-                    {"".join(self.xlsx_sheets[x]["sheetData"][z]["cells"])}
-                    </row>"""
-                for z in range(len(self.xlsx_sheets[x]["sheetData"]))
-            )
+            sheet_data = ""
+            for z in range(len(self.xlsx_sheets[x]["sheetData"])):
+                sheet_data += f'\t<row r="{z+1}" '
+                sheet_data += f'\t customHeight="1" ht="{self.xlsx_sheets[x]["sheetData"][z]["height"]}" '
+                sheet_data += f"""\n\t{"hidden='true'" if self.xlsx_sheets[x]["sheetData"][z]["height"] == 0 else "hidden='false'"} """
+                sheet_data += f'\n\toutlineLevel="{self.xlsx_sheets[x]["sheetData"][z]["outline_level"]}" '
+                sheet_data += ' collapsed="false" >'
+                sheet_data += f'{"".join(self.xlsx_sheets[x]["sheetData"][z]["cells"])}'
+                sheet_data += "</row>"
 
             merges = "".join(self.xlsx_sheets[x]["spans"])
             if merges:
@@ -229,11 +200,12 @@ class Q2PrinterXlsx(Q2Printer):
 
         sheet_row = {}
         for row in range(row_count):  # вывод - по строкам
-            # sheet_row["height"] = (
-            #     rows_section["real_heights"][row] if rows_section["real_heights"][row] else num(0.7)
-            # ) * points_in_cm
+            height = rows_section["row_height"][row]
+            if row in rows_section["auto_height_rows"]:
+                height = 0
+
+            sheet_row["height"] = height * points_in_cm
 
-            sheet_row["height"] = rows_section["max_cell_height"][row] * points_in_cm
             sheet_row["cells"] = []
             sheet_row["outline_level"] = outline_level
             for col in range(self._columns_count):  # цикл по клеткам строки
@@ -243,7 +215,6 @@ class Q2PrinterXlsx(Q2Printer):
                     continue
                 cell_address = self.get_cell_address(self.sheet_current_row, col)
                 cell_data = rows_section.get("cells", {}).get(key, {})
-                cell_text = cell_data.get("data", "")
 
                 row_span = cell_data.get("rowspan", 1)
                 col_span = cell_data.get("colspan", 1)
@@ -254,7 +225,7 @@ class Q2PrinterXlsx(Q2Printer):
                     # cell_data["style"] = cell_style
 
                 self.make_image(cell_data, row, col)
-                cell_xml = self.make_xlsx_cell(cell_address, cell_style, cell_text, cell_data)
+                cell_xml = self.make_xlsx_cell(cell_address, cell_style, cell_data)
                 sheet_row["cells"].append(cell_xml)
                 if row_span > 1 or col_span > 1:
                     merge_str = ":".join(
@@ -270,7 +241,7 @@ class Q2PrinterXlsx(Q2Printer):
                                 self.sheet_current_row + span_row, span_col + col
                             )
                             spanned_cells[f"{span_row+row},{span_col+col}"] = self.make_xlsx_cell(
-                                cell_address, cell_style, ""
+                                cell_address, cell_style
                             )
             self.current_sheet["sheetData"].append(dict(sheet_row))
             self.sheet_current_row += 1
@@ -288,12 +259,54 @@ class Q2PrinterXlsx(Q2Printer):
             len(self.cell_xfs),
             "".join("\n%s" % style for style in self.cell_xfs),
         )
+        fills = """\n<fills count="%s">%s</fills>""" % (
+            len(self.fills),
+            "\n".join(self.fills),
+        )
+
+        self.convert_num_fmt()
+        num_fmts = f"""<numFmts count="{len(self.num_fmts)}">
+                        {''.join(['<numFmt numFmtId="%s" formatCode="%s"/>' % (index+164, fmt)
+                        for index, fmt in enumerate(self.num_fmts)])}
+                    </numFmts>
+        """
 
         zipf.writestr("xl/styles.xml", (xlsx_parts["xl/styles.xml"] % locals()).encode("utf8"))
 
+    def convert_num_fmt(self):
+        for idx, fmt in enumerate(self.num_fmts):
+            if fmt.startswith("N"):
+                dec = int_(_.group()) if (_ := reNumber.search(fmt)) else None
+                if dec is not None:
+                    efmt = "0."
+                    efmt += "0" * dec
+                    zero = efmt if "Z" in fmt else ""
+                    self.num_fmts[idx] = f"{efmt};-{efmt};{zero};@"
+                else:
+                    if "Z" in fmt:
+                        self.num_fmts[idx] = "General"
+                    else:
+                        self.num_fmts[idx] = "[=0]" ";General"
+            elif fmt.startswith("F"):
+                dec = int_(_.group()) if (_ := reNumber.search(fmt)) else None
+                if dec is not None:
+                    efmt = "#,###0."
+                    efmt += "0" * dec
+                    zero = efmt if "Z" in fmt else ""
+                    self.num_fmts[idx] = f"{efmt};-{efmt};{zero};@"
+                elif dec is None:
+                    efmt = "#,###0.############"
+                    zero = efmt if "Z" in fmt else ""
+                    self.num_fmts[idx] = f"{efmt};-{efmt};{zero};@"
+                else:
+                    if "Z" in fmt:
+                        self.num_fmts[idx] = "General"
+                    else:
+                        self.num_fmts[idx] = "[=0]" ";General"
+
     def get_cell_xf_id(self, style, numFmtId=0):
         border = f'borderId="{self.get_cell_borders(style)}"'
-        fill = f'fillId="{0}"'
+        fill = f'fillId="{self.get_fills_id(style)}"'
         font = f'fontId="{self.get_font_id(style)}"'
         num_fmt = f'numFmtId="{numFmtId}"'
         align = self.get_cell_align(style)
@@ -306,17 +319,37 @@ class Q2PrinterXlsx(Q2Printer):
         return xf_id
 
     def get_font_id(self, style):
-        font_size = num(style["font-size"].replace("pt", ""))
+        # font_size = num(str(style["font-size"]).replace("pt", ""))
+        font_size = num(str(style["font-size"]).replace("pt", "")) * num(0.93)
         font_family = style["font-family"]
         font_weight = "<b/>" if style.get("font-weight", "") == "bold" else ""
-        font_style = """<name val="%s"/> <sz val="%s"/>%s""" % (font_family, font_size, font_weight)
+        font_color = f'<color rgb="{css_color_to_rgb(color)}"/>' if (color := style.get("color", "")) else ""
+        font_style = f'<name val="{font_family}"/> <sz val="{font_size}"/> {font_color} {font_weight}'
         if font_style not in self.fonts:
             self.fonts.append(font_style)
-        #     font_id = len(self.fonts) - 1
-        # else:
         font_id = self.fonts.index(font_style)
         return font_id
 
+    def get_fills_id(self, style):
+        color = style.get("background")
+        if color is None:
+            return 0
+        fill = (
+            '<fill><patternFill patternType="solid">'
+            f'<fgColor rgb="{css_color_to_rgb(color)}"/>'
+            f'<bgColor rgb="{css_color_to_rgb(color)}"/>'
+            "</patternFill></fill>"
+        )
+
+        if fill not in self.fills:
+            self.fills.append(fill)
+        return self.fills.index(fill)
+
+    def get_num_fmt_id(self, numFormat):
+        if numFormat not in self.num_fmts:
+            self.num_fmts.append(numFormat)
+        return self.num_fmts.index(numFormat) + 164
+
     def make_image(self, cell_data, row, col):
         for x in cell_data.get("images", []):
             width, height, imageIndex = self.prepare_image(x, cell_data.get("width"))
@@ -332,82 +365,49 @@ class Q2PrinterXlsx(Q2Printer):
             tmp_drawing["_width"] = int(width)
             self.current_sheet["drawing"].append(tmp_drawing)
 
-    def make_xlsx_cell(self, cell_address, cell_style, cell_text, cell_data={}):
-        if cell_data.get("numFmtId"):
-            xf_id = self.get_cell_xf_id(cell_style, cell_data.get("numFmtId"))
-            return f"""\n\t<c r="{cell_address}"
-                            s="{xf_id}"
-                            t="n">
-                    <v>{cell_data.get("xlsx_data")}</v>
-                    \n\t</c>"""
+    def make_xlsx_cell(self, cell_address, cell_style, cell_data: dict = {}):
+        raw_text = cell_data.get("xlsx_data", "")
+        cell_format = cell_data.get("format", "")
 
-        fontsizemod = fontsize = cell_style["font-size"].replace("pt", "")
-        fontfamily = cell_style["font-family"]
+        isNumber = reDecimal.match(raw_text)
+        if isNumber and (cell_format or cell_style.get("text-align") == "right"):
+            numFmtId = self.get_num_fmt_id(cell_format) if cell_format else 0
 
-        cell_content = []
-        cell_text = cell_text.replace("\n", "")
-        cell_text = cell_text.replace("\r", "")
-        cell_text = reMultiSpaceDelete.sub(" ", cell_text)
-        cell_text = reHtmlTagBr.sub("\n", cell_text)
-        if re.findall(r"\<(\/*b)|(br/)|(font)|(u)|(i)\>", cell_text, re.IGNORECASE):
-            bold = ""
-            ital = ""
-            undl = ""
-            for x in cell_text.split("<"):
-                if ">" in x:
-                    stl = x.split(">")[0].upper().strip().replace(" ", "")
-                    if "B" == stl:
-                        bold = "<b/>"
-                    elif "/B" == stl:
-                        bold = ""
-                    elif "I" == stl:
-                        ital = "<i/>"
-                    elif "/I" == stl:
-                        ital = ""
-                    elif "U" == stl:
-                        undl = """<u/>"""
-                    elif "/U" == stl:
-                        undl = ""
-                    elif "/FONT" in stl:
-                        fontsizemod = fontsize
-                    # elif "FONTSIZE=" in stl:
-                    #     fontsizemod = grid.getFontSizeMod(fontsize / 2, stl.split("=")[1])
-                    x = x.split(">")[1]
-                if x.strip():
-                    cell_content.append(
-                        f"""<r>
-                                <rPr>
-                                    <rFont val="{fontfamily}"/>
-                                    <sz val="{fontsizemod}"/>{bold}{ital}{undl}
-                                </rPr><t xml:space="preserve">{x}</t>
-                            </r>"""
-                    )
-        elif cell_text != "":
-            cell_content.append(f'<t xml:space="preserve">{cell_text}</t>')
-
-        xf_id = self.get_cell_xf_id(cell_style)
-
-        if cell_content:
-            cell_content = "".join(cell_content)
-            if cell_content not in self.sharedStrings:
-                self.sharedStrings.append(cell_content)
-                shared_strings_id = len(self.sharedStrings) - 1
-            else:
-                shared_strings_id = self.sharedStrings.index(cell_content)
-            return f"""\n\t<c r="{cell_address}"
-                            s="{xf_id}"
-                            t="s">
-                    \n\t\t<v>{shared_strings_id}</v>
-                    \n\t</c>"""
-            # return (
-            #     f'''\n\t<c r="{cell_address}"
-            #                 s="2"
-            #                 t="n">
-            #         <v>123</v>
-            #         \n\t</c>'''
-            # )
+            xf_id = self.get_cell_xf_id(cell_style, numFmtId)
+            return f"""\n\t <c r="{cell_address}" s="{xf_id}" t="n">
+                                    <v>{raw_text}</v>
+                                </c>"""
         else:
-            return f'\n\t<c r="{cell_address}" s="{xf_id}"/> '
+            xf_id = self.get_cell_xf_id(cell_style)
+            cell_text = cell_data.get("data", "")
+            # Normalize cell text
+            cell_text = (cell_text or "").replace("\r", "").replace("\n", "")
+            cell_text = reMultiSpaceDelete.sub(" ", cell_text)
+            cell_text = reHtmlTagBr.sub("\n", cell_text)
+
+            # Parse inline formatting
+            fontsize = str(cell_style.get("font-size", "10pt")).replace("pt", "")
+            fontfamily = cell_style.get("font-family", "Calibri")
+            parser = RichTextParser(fontfamily, fontsize)
+            parser.feed(cell_text, cell_style)
+            runs = parser.get_runs()
+
+            if len(runs) == 1:
+                cell_content = f"<t>{cell_text}</t>"
+            else:
+                cell_content = "".join(runs)
+
+            if cell_content:
+                if cell_content not in self.sharedStrings:
+                    self.sharedStrings.append(cell_content)
+                    shared_strings_id = len(self.sharedStrings) - 1
+                else:
+                    shared_strings_id = self.sharedStrings.index(cell_content)
+                return f"""\n\t <c r="{cell_address}" s="{xf_id}" t="s">
+                                    <v>{shared_strings_id}</v>
+                                </c>"""
+            else:
+                return f'\n\t<c r="{cell_address}" s="{xf_id}"/>'
 
     def get_cell_align(self, cell_style):
 
@@ -436,15 +436,24 @@ class Q2PrinterXlsx(Q2Printer):
                 horizontal = f""" horizontal="left" indent="{int(round((num(padding[3])) / num(0.25)))}" """
         return f'\n\t<alignment {horizontal} {vertical} wrapText="true"/>\n'
 
-    def get_cell_borders(self, style):
-        border_width = style["border-width"].split(" ")
+    def get_cell_borders(self, cell_style):
+        border_width = cell_style["border-width"].split(" ")
         while len(border_width) < 4:
             border_width += border_width
         border = []
-        for index, side in enumerate(("left", "right", "top", "bottom")):
-            if int_(border_width[index]):
-                bw = self.get_border_width(border_width[index])
-                border.append(f'<{side} style="{bw}"><color auto="1"/></{side}>')
+
+        border_color = 'auto="1"'
+        if color := cell_style.get("border-color"):
+            border_color = f' rgb="{css_color_to_rgb(color)}"'
+
+        border_width_dict = {
+            side: int_(border_width[index]) for index, side in enumerate(("top", "right", "bottom", "left"))
+        }
+
+        for side in ("left", "right", "top", "bottom"):
+            if int_(border_width_dict[side]):
+                bw = self.get_border_width(border_width_dict[side])
+                border.append(f'<{side} style="{bw}"><color {border_color}/></{side}>')
         border.append("<diagonal/>")
 
         borders = "\n".join(border)
